{{ TOBI_3_v47.spin

        TOBI - The Tool Bot

        TheGrue
        John Harris
        http://letsmakerobots.com/node/7025

        TOBI - Recieve Data from XBee RoBo Remote and control motors
        Added PING))) Sensor array to detect objects in the robots path
        Added 2 Chasis mounted PING)))'s at 45 degree angles
        Added 2 front bumper switches
        Updated the Object detection to overide the RF signals and steer away from objects
        Fixed the OverRide fuction
        Changed XBee my Address for the Parallax Expo to 4
        Attempted Thermopile readings
}}

CON

  _xinfreq = 5_000_000          ' 5 MHz cyrstal
  _clkmode = xtal1 + pll16x     ' use crystal x 16 (sys clock = 80 MHz)
  
' ---Pin Assignments as indicated by the "p_" beginning ---
  p_LCD         = 0             ' Front LCD
  p_SW          = 8             ' Momentary Switch on front Panel
  p_Green       = 9             ' Green LED on front Panel
  p_Red         = 10            ' Red LED on front Panel

  p_Hb25Pwr     = 24            ' Power to both HB-25's
  p_LHb25       = 1             ' Left HB25 Logical Input
  p_RHb25       = 2             ' Right HB25 Logical Input

  p_LServo      = 20            ' Left Sensor Array Servo
  p_RServo      = 21            ' Right Sensor Array Servo
  p_LPing       = 22            ' Left PING))) in Left Sensor Array
  p_CPing       = 3             ' Center PING))) above Thermal Sensor
  p_RPing       = 23            ' Right PING))) in Right Sensor Array
  p_LAPing      = 4             ' Left Angle Ping mounted on chassis
  p_RAPing      = 5             ' Right Angle Ping mounted on chassis

  p_LBumper     =12             ' Left front bumper switch
  p_RBumper     =11             ' Right frond bumper switch
  p_BBumper     =13             ' Back bumper (not installed yet)

  p_XBTx        = 14            ' XBee Digital Input DIN
  p_XBRx        = 15            ' XBee Digital Output DOUT

  p_SCL         = 16            ' I2C SCL for Thermal array
  p_SDA         = 17            ' I2C SDA for Thermal array

  p_Encoders    = 18            ' Both wheel encoders

'' --- General Constants ---

' --- LCD Constants ---
  LCD_LINES     = 4
' LCD_BAUD      = 2_400         ' Too slow but works
  LCD_BAUD      = 9_600         ' Works great
' LCD_BAUD      = 19_200        ' Doesn't work

  '-- XBee --
  XB_Baud       = 9600          ' XBee Baud Rate
  MY_Addr       = 4             ' XBee MY Address of TOBI
  DL_Addr       = 0             ' XBee Send to RoBo Remote

  '-- Servo Constants --
  RampRate      = 400           ' Ramp Rate: Good Ramp Rate for drive motors
  ServoTime     = 3             ' Time Divider for servo sweep

  '-- Obsticle Constants --
  ObFwd         = 5
  ObAngl        = 3
  ObSd          = 1
  ObBkUp        = 3

  
OBJ

  LCD:          "Debug_Lcd"                             'LCD Display
  XB:           "XBee_Object"                           'XBee Communication Object
  Ping:         "Ping"                                  'Ping Device
  Servo:        "Servo32v7"                             'Servo Object for Sensor Array
  Drive:        "Servo32v7"                             'Servo Object for motors
  tpa81:         "TPA81Object"                           'TPA81 Thermopile Object
  vp:           "Conduit"                               'Viewport Debugger Object

VAR

  long  dataIn, duration, Forward, Turn, Speed, OverRide, Reverse, Left_Dr, Right_Dr, PulseMin, PulseMax, TurnAdjust, LLimit, HLimit, FWD, TRN, SensorStatus, PngLeft, PngLeftAngle, PngLeftFront, PngCenter, PngRightFront, PngRightAngle, PngRight, ObForward, ObTurn, LeftBumper, RightBumper
  long Therm[8]
  long Stack[100]
  
PUB Main

'   Diagnostics                                         ' Set ViewPort up to connect for Diagnostics - comment out when not used

' --- Set Pins as Inputs or Outputs ---
  dira[p_LCD]~~
  dira[p_Green]~~
  dira[p_Red]~~
  dira[p_Hb25Pwr]~~
  dira[p_LHb25]~~
  dira[p_RHb25]~~
  dira[p_LServo]~~
  dira[p_RServo]~~
  dira[p_XBTx]~~
  dira[p_XBRx]~
  dira[p_SW]~
  dira[p_LBumper]~
  dira[p_RBumper]~
  dira[p_BBumper]~


' --- Initialization ---
  outa[p_Green]~~                                       ' Ligh Green Power LED on front Panel

' --- Servo Set Pings to Sides ---
  duration := 30        
  repeat until --duration == 0                          ' Short Loop for Side Position of Servos
    outa[p_LServo..p_RServo]~~
    waitcnt(clkfreq/10000*19 + cnt)
    outa[p_LServo..p_RServo]~
    waitcnt(clkfreq/1000*20 + cnt)
    
' --- Initialize LCD Display ---
  LCD.init(p_LCD, LCD_BAUD, LCD_LINES)
  waitcnt(clkfreq/2 +cnt)
  LCD.cursor(0)
  LCD.cls
  LCD.backLight(true)
  LCD.display(1)
  LCD.cursor(0)
  LCD.cls
  LCD.cls
  LCD.cls
  LCD.gotoxy(0, 0)
  LCD.gotoxy(0, 0)
  LCD.str(@TTTB)
  LCD.str(@Line)
  LCD.putc(LCD#LCD_CR)
  LCD.str(string("Booting Up Robot"))
  LCD.cursor(0)
  LCD.backLight(true)

' --- Servo Start ---
  SERVO.Start                                           ' Start servo's
  DRIVE.Start
  DRIVE.Ramp                                            '<-OPTIONAL 'Start Background Ramping
   
' --- Initialize XBee ---
  XB.Start(p_XBRx, p_XBTx ,0, XB_Baud)
  XB.AT_Init
  XB.AT_ConfigVal(string("ATMY"), MY_Addr)   
  XB.AT_ConfigVal(string("ATDL"),DL_Addr)


' --- Initialize Variables ---
  PulseMin   := 1200            ' PulseMin = 1000-1400 Range - 1000 is full speed reverse and 1400 is slow reverse
  PulseMax   := 1750            ' PulseMax = 1600-2000 Range - 2000 is full speed forward and 1600 is slow forward
  TurnAdjust := 20              ' TurnAdjust = 5-50 - 5 is slow wide turns and 50 is sharp fast turns
  Left_Dr    := 1500
  Right_Dr   := 1500

' --- Init Sensor Array ---
  Cognew(SensorArray, @Stack)

' --- Init TPA81 Thermopile ---
  tpa81.New(p_SDA, p_SCL)                               ' start the Cog

' --- Main Routine ---
  MainMission

PUB MainMission | count
' repeat until ina[p_SW]                                ' Wait for switch to be pushed 
  LCD.clrln(3)

  repeat
    XB.RxFlush                                          ' clear data in buffer
    outa[p_Green]~~
     
       DataIn := XB.RxTime(1500)                        ' wait for incoming byte
       If DataIn == -1                                  ' if no data after 1.5 seconds
           SERVO.Set(p_RHb25, 1500)                     ' stop drive servos
           SERVO.Set(p_LHb25, 1500)
           SensorStatus := 0
           LCD.gotoxy(0, 0)
           LCD.str(@TTTB)
           LCD.str(@Line)
           LCD.str(@RBRM)
           LCD.str(@NF)
           repeat 5                                     ' blink red LED 5 times
             outa[p_Red]~~
             XB.delay(50)
             outa[p_Red]~
             XB.delay(50)  
          outa[p_Hb25Pwr]~                              ' Turn on HB-25's if not already on

    case DataIn                                         ' test acccepted data

       "h":
           SERVO.Set(p_RHb25, 1500)                     ' stop drive servos
           SERVO.Set(p_LHb25, 1500)
           SensorStatus := 0
           LCD.clrln(2)
           LCD.clrln(3)
           LCD.gotoxy(0, 0)
           LCD.str(@TTTB)
           LCD.str(@Line)
           LCD.gotoxy(0, 3)
           LCD.str(@SCP)
           repeat 5                                     ' blink red LED 5 times
             outa[p_Green]~~
             XB.delay(50)
             outa[p_Green]~
             XB.delay(50)  
          outa[p_Hb25Pwr]~                              ' Turn on HB-25's if not already on


      "d":                                              ' if drive data
           outa[p_Hb25Pwr]~~                            ' Turn on HB-25's if not already on
           Forward     := XB.RxDEC                      ' get right and left drive
           Turn        := XB.RxDEC
           Speed       := XB.RxDEC                      ' get right and left drive
           OverRide    := XB.RxDEC
           Reverse     := XB.RxDEC
           If OverRide == 0
             SensorStatus := 1
           If OverRide == 1
             SensorStatus := 0
           ObsticleAvoidance
           LCD.gotoxy(0, 0)
           LCD.str(@TTTB)
           LCD.str(@Line)
           LCD.gotoxy(0, 2)
           LCD.str(String("FWD: "))
           LCD.dec(Forward)
           LCD.str(String(" "))
           LCD.gotoxy(10, 2)
           LCD.str(String("TRN: "))
           LCD.dec(Turn)
           LCD.str(String(" "))
           LCD.str(String(13))
           LCD.str(@RT)
           LCD.dec(Right_dr)
           LCD.str(@LFT)
           LCD.dec(Left_dr)

      "f":
'        repeat
           outa[p_Hb25Pwr]~~                            ' Turn on HB-25's if not already on
           SensorStatus := 0
           Reverse := 0
           LCD.gotoxy(0, 0)
'          LCD.str(@TTTB)
'          LCD.str(@Line)
'          LCD.clrln(2)
'          LCD.clrln(3)
'          LCD.gotoxy(0, 3)
           LCD.cls
           LCD.str(@FLW)
           LCD.gotoxy(0, 1)
'          ThermoSensor
             repeat Count from 0 to 7                       ' output 8 array pixels
               LCD.dec(tpa81.AmbientTemperature(15)) ' ambient temperature at spos

               LCD.dec(tpa81.GetValue(15, Count))
               LCD.str(@Spc)
           

      "t":
           SensorStatus := 0
           LCD.cls
           LCD.gotoxy(0, 0)
           LCD.str(@TL1)
           LCD.str(@TL2)
           LCD.str(@TL3)
           LCD.str(@TL4)
           waitcnt(clkfreq * 5 + cnt)

PUB TestCollision

  ' To test if robot collided with an object
  ' Variables: LeftBumper, RightBumper

  if ina[p_LBumper] == 1 and ina[p_RBumper] == 1
   DRIVE.set(p_LHb25,1400)
   DRIVE.set(p_RHb25,1400)
   waitcnt(clkfreq +cnt)
   
  Elseif ina[p_LBumper] == 1
   DRIVE.set(p_LHb25,1400)
   DRIVE.set(p_RHb25,1350)
   waitcnt(clkfreq +cnt)
   
  Elseif ina[p_RBumper] == 1
   DRIVE.set(p_LHb25,1350)
   DRIVE.set(p_RHb25,1400)
   waitcnt(clkfreq +cnt)
   
PUB ObsticleAvoidance

'   Constantants: ObFwd, ObAngl, ObSd, ObBkUp,
'   Variables: SensorStatus, Reverse, Forward, Turn, ObTurn
 
'                    PngLeftFront   PngCenter   PngRightFront 
'             PngLeftAngle,                           PngRightAngle
'           PngLeft,                                         PngRight

  '--Determine the best way to avoid any obsticles--

  ' Tesing for Collision
  If ina[p_LBumper] == 1 or ina[p_RBumper] == 1
    TestCollision
    Return

  ' Robot moving in forward or caster first mode
  If Reverse == 0

    If Forward => 1

      if PngLeftFront =< 6 AND PngRightFront =< 6
        Forward := 0
        if PngLeftFront =< 4 AND PngRightFront =< 4 
          Forward := Forward - ObBkUp
        Return
       
      If PngCenter =< 12
        Forward := 0
        If PngCenter =< 8
            Forward := Forward - ObBkUp 

      ' Reset ObTurn to 0
      ObTurn := 0
      
      ' Forward
      If PngLeftFront =< 12
        ObTurn := ObTurn + ObFwd

      If PngRightFront =< 12
        ObTurn := ObTurn - ObFwd

      ' Angles
      If PngLeftAngle =< 12
        ObTurn := ObTurn + ObAngl

      If PngRightAngle =< 12
        ObTurn := ObTurn - ObAngl
      
      ' Sides
'     If PngLeft =< 8
'       ObTurn := ObTurn + ObSd

'     If PngRight =< 8
'       ObTurn := ObTurn - ObSd
      
      '--For Testing the turn--
      If ObTurn <> 0
        Turn := ObTurn
         
'   Robot moving in Reverse mode Wheels first
  Elseif Reverse == 1

      ' Reset ObTurn to 0
      ObTurn := 0

      ' Sides
      If PngLeft =< 8
        ObTurn := ObTurn + ObSd

      If PngRight =< 8
        ObTurn := ObTurn - ObSd

      '--For Testing the turn--
      If ObTurn <> 0
        Turn := 0

  ComputeDrive


PUB ComputeDrive  

  ' Set Speed
  If Speed == 0                                       ' Middle slider switch is off, run at half speed
    PulseMin := 1200                                  ' PulseMin = 1000-1400 Range - 1000 is full speed reverse and 1400 is slow reverse
    PulseMax := 1750                                  ' PulseMax = 1600-2000 Range - 2000 is full speed forward and 1600 is slow forward     
   
  else                                                ' Else if middle slider is on, run at top speed  
    PulseMin := 1000                                  ' PulseMin = 1000-1400 Range - 1000 is full speed reverse and 1400 is slow reverse
    PulseMax := 2000                                  ' PulseMax = 1600-2000 Range - 2000 is full speed forward and 1600 is slow forward
   
  LLimit := TurnAdjust + PulseMin
  HLimit := TurnAdjust + PulseMax

  ' Find common forward speed 
  FWD := (LLimit #> ((Forward * 50) + 1500)<# HLimit) 'multiply by 50 add the result to 1500 center pulse and limit the minimum value to 1050
   
  ' If in Reverse, Limit speed
  if Reverse == 1
     TurnAdjust := 10
  Else
     TurnAdjust := 20
   
  ' Figure out Turn from Y-axis
  TRN := Turn * TurnAdjust
   
   
  ' Use Turn VAR to figure out Left/RightPulse
  '   Note the equation for the RightPulse. I am using HB-25's where a 2ms pulse moves them both forward.
  '     If you wish to use this for continuous rotation servo's change the (-) minus sign to a (+) plus
  '     sign to generate the correct pulses.
  FWD :=      PulseMin #> FWD <# PulseMax
  Left_Dr := PulseMin #> FWD + TRN <# PulseMax 'Write result of Forward and turn back to the Variable LeftPulse
  Right_Dr := PulseMin #> FWD - TRN <# PulseMax 'Write result of Forward and turn back to the Variable RightPulse

'  ---Send Drive Commands---
  'SetRamp(Pin, Width,Delay)<-- 100 = 1 sec 6000 = 1 min            
   DRIVE.SetRamp(p_RHb25, Right_dr, RampRate)           ' drive servos based on data
   DRIVE.SetRamp(p_LHb25, Left_dr, RampRate)
   
PUB SensorArray

'   PngLeft, PngLeftAngle, PngLeftFront, PngCenter, PngRightFront, PngRightAngle, PngRight, LeftBumper, RightBumper
    Dira[p_RServo] := Dira[p_LServo] := 1
    
 Repeat

 
  if SensorStatus == 1 
   
    If Reverse == 0
       
       '-- Set Servo's forward
       Servo.set(p_LServo,1000)
       Servo.set(p_RServo,1000)
       
       PngCenter      := Ping.Inches(p_CPing)
       PngLeftAngle   := Ping.Inches(p_LAPing)
       PngRightAngle  := Ping.Inches(p_RAPing)
       PngLeftFront   := Ping.Inches(p_LPing)
       PngRightFront  := Ping.Inches(p_RPing)

    Elseif Reverse == 1
    
        '-- Set Servo's to sides
       Servo.set(p_LServo,2000)
       Servo.set(p_RServo,2000)
       
       
      PngCenter := Ping.Inches(p_CPing)
      PngLeft := Ping.Inches(p_LPing)
      PngRight := Ping.Inches(p_RPing)
      waitcnt(clkfreq /10 + cnt)

  Elseif SensorStatus == 0

      '-- Set Servo's to sides
       Servo.set(p_LServo,2000)
       Servo.set(p_RServo,2000)

       PngCenter      := 30
       PngLeftAngle   := 30
       PngRightAngle  := 30
       PngLeftFront   := 30
       PngRightFront  := 30
  
PUB ThermoSensor | Count

    repeat Count from 0 to 7                                ' output 8 array pixels
      Therm[Count] := tpa81.GetValue(1, Count)
    

       
PUB Diagnostics
  vp.config(string("start:terminal"))
  vp.config(string("var:Forward, Turn, Speed, OverRide, Reverse, Left_Dr, Right_Dr, PulseMin, PulseMax, TurnAdjust, LLimit, HLimit, FWD, TRN, SensorStatus, PngLeft, PngLeftAngle, PngLeftFront, PngCenter, PngRightFront, PngRightAngle, PngRight, ObForward, ObTurn, LeftBumper, RightBumper"))
  vp.config(string("text:timescale=10ms"))
   
  vp.share(@OverRide,@ObTurn)

DAT

TTTB          byte      "TOBI - The Tool Bot ", 0
Line          byte      "--------------------", 0
TL1           byte      "TTTTT  OO  BBB  III ", 0
TL2           byte      "  T   O  O B__B  I  ", 0
TL3           byte      "  T   O  O B  B  I  ", 0
TL4           byte      "  T    OO  BBBB III", 0
SCP           byte      "     Connected",0
RT            byte      " RT: ",0
LFT           byte      " LFT: ",0
RBRM          byte      "    RoBo Remote     ",0
NF            byte      "     Not Found      ", 0
Blank         byte      "                    ", 0
FLW           byte      "   Follow Me Mode", 0
Spc           byte      "  ",0